from vital_ai_vitalsigns.ontology.ontology import Ontology
from vital_ai_vitalsigns.service.graph.binding import Binding


class ConstructQuery:
    def __init__(self, namespace_list: list[Ontology], binding_list: list[Binding], query: str,
                 root_binding: str | None = None,
                 limit: int | None = None, offset: int | None = None):
        self.namespace_list = namespace_list
        self.binding_list = binding_list
        self.query = query
        self.root_binding = root_binding
        self.limit = limit
        self.offset = offset




from typing import List, TypeVar, Type, Tuple
from ai_haley_kg_domain.model.KGInteraction import KGInteraction
from ai_haley_kg_domain.model.KGNode import KGNode
from vital_ai_vitalsigns.query.result_list import ResultList
from vital_ai_vitalsigns.service.vital_namespace import VitalNamespace
from vital_ai_vitalsigns.service.vital_service import VitalService
from vital_ai_vitalsigns.service.vital_service_status import VitalServiceStatus


G = TypeVar('G', bound='GraphObject')
KGN = TypeVar('KGN', bound='KGNode')

# notes:
# define primary vector for kg objects and secondary vector
# for the type of the object
# use naming convention _vector and _type_vector
# see: https://weaviate.io/developers/weaviate/config-refs/schema/multi-vector


class KGraphService:
    def __init__(self, vital_service: VitalService):
        self.vital_service = vital_service

    def get_graph(self, graph_uri: str) -> VitalNamespace:
        return self.vital_service.get_graph(graph_uri)

    def list_graphs(self) -> List[VitalNamespace]:
        return self.vital_service.list_graphs()

    def check_create_graph(self, graph_uri: str) -> bool:
        return self.vital_service.check_create_graph(graph_uri)

    def create_graph(self, graph_uri: str) -> bool:
        return self.vital_service.create_graph(graph_uri)

    def delete_graph(self, graph_uri: str) -> bool:
        return self.vital_service.delete_graph(graph_uri)

    def purge_graph(self, graph_uri: str) -> bool:
        return self.vital_service.purge_graph(graph_uri)

    def get_graph_all_objects(self, graph_uri: str, limit=100, offset=0) -> ResultList:
        return self.vital_service.get_graph_all_objects(graph_uri, limit=limit, offset=offset)

    def insert_object(self, graph_uri: str, graph_object: G) -> VitalServiceStatus:
        return self.vital_service.insert_object(graph_uri, graph_object)

    def insert_object_list(self, graph_uri: str, graph_object_list: List[G]) -> VitalServiceStatus:
        return self.vital_service.insert_object_list(graph_uri, graph_object_list)

    def update_object(self, graph_object: G, graph_uri: str) -> VitalServiceStatus:
        return self.vital_service.update_object(graph_object, graph_uri)

    def update_object_list(self, graph_object_list: List[G], graph_uri: str) -> VitalServiceStatus:
        return self.vital_service.update_object_list(graph_object_list, graph_uri)

    def get_object(self, object_uri: str, graph_uri: str | None = None) -> G:
        return self.vital_service.get_object(object_uri, graph_uri)

    def get_object_list(self, object_uri_list: List[str], graph_uri: str | None = None) -> ResultList:
        return self.vital_service.get_object_list(object_uri_list, graph_uri)

    def delete_object(self, object_uri: str, graph_uri: str | None = None) -> VitalServiceStatus:
        return self.vital_service.delete_object(object_uri, graph_uri)

    def delete_object_list(self, object_uri_list: List[str], graph_uri: str | None = None) -> VitalServiceStatus:
        return self.vital_service.delete_object_list(object_uri_list, graph_uri)

    def filter_query(self, graph_uri: str, sparql_query: str) -> ResultList:
        return self.vital_service.filter_query(graph_uri, sparql_query)

    def query(self, graph_uri: str, sparql_query: str) -> ResultList:
        return self.vital_service.query(sparql_query, graph_uri)

    # Tuple[binding name, property uri]
    def query_construct(self, graph_uri: str, sparql_query: str, binding_list: List[Tuple[str, str]]) -> ResultList:
        pass

    def get_interaction_list(self, graph_uri: str, limit=100, offset=0) -> ResultList:
        pass

    def get_interaction_graph(self, graph_uri: str, interaction: KGInteraction, limit=100, offset=0) -> ResultList:
        pass

    def get_interaction_frames(self, graph_uri: str, interaction: KGInteraction, limit=100, offset=0) -> ResultList:
        pass

    def get_interaction_nodes(self, graph_uri: str, interaction: KGInteraction, kgnode_type: Type[KGN], limit=100, offset=0) -> ResultList:
        pass

    def get_frame(self, graph_uri: str, frame_uri: str, limit=100, offset=0) -> ResultList:
        pass

    def get_frames(self, graph_uri: str, frame_uri_list: List[str], limit=100, offset=0) -> ResultList:
        pass

    def get_frame_id(self, graph_uri: str, frame_id: str, limit=100, offset=0) -> ResultList:
        pass

    def get_frames_id(self, graph_uri: str, frame_id_list: List[str], limit=100, offset=0) -> ResultList:
        pass

    def get_frames_root(self, graph_uri: str, root_uri: str, limit=100, offset=0) -> ResultList:
        pass

    def get_graph_objects_type(self, graph_uri: str, class_uri: str, include_subclasses=True, limit=100, offset=0) -> ResultList:
        pass

    def get_graph_objects_tag(self, graph_uri: str, kg_graph_uri: str, limit=100, offset=0) -> ResultList:
        pass

    def delete_frame(self, graph_uri: str, frame_uri: str) -> VitalServiceStatus:
        pass

    def delete_frames(self, graph_uri: str, frame_uri_list: List[str]) -> VitalServiceStatus:
        pass

    def delete_frame_id(self, graph_uri: str, frame_id: str) -> VitalServiceStatus:
        pass

    def delete_frames_id(self, graph_uri: str, frame_id_list: List[str]) -> VitalServiceStatus:
        pass

    def delete_graph_objects_tag(self, graph_uri: str, kg_graph_uri: str) -> VitalServiceStatus:
        pass



from typing import List
from vital_ai_vitalsigns.ontology.ontology import Ontology
from vital_ai_vitalsigns.service.graph.binding import Binding
from kgraphservice.kgslot_criterion import KGSlotCriterion
from kgraphservice.query.construct_query import ConstructQuery


class FrameQueryGenerator:

    @classmethod
    def generate_uri_query(cls, frame_uri: str) -> ConstructQuery:

        namespace_list = cls.get_default_namespace_list()

        binding_list = [
            Binding("?frame", "urn:hasFrame"),
            Binding("?slotEdge", "urn:hasSlotEdge"),
            Binding("?slot", "urn:hasSlot")
        ]

        slot_class_list = cls.get_slot_class_list()

        frame_query = f"""
            {{
                VALUES ?frameURI {{
                    <{frame_uri}>
                }}
                
                VALUES ?slotDataType {{
                    {slot_class_list}
                }}

                BIND(?frameURI AS ?frame)

                ?frame rdf:type haley-ai-kg:KGFrame .

                ?slotEdge vital-core:hasEdgeSource ?frame .
                ?slotEdge vital-core:hasEdgeDestination ?slot .
                ?slotEdge rdf:type haley-ai-kg:Edge_hasKGSlot .

                ?slot rdf:type ?slotType .
            }}

            FILTER(?slotType IN (?slotDataType))
        """

        construct_query = ConstructQuery(namespace_list, binding_list, frame_query)

        return construct_query

    @classmethod
    def generate_uri_list_query(cls, frame_uri_list: List[str]) -> ConstructQuery:

        frame_uri_values = "\n".join([f"<{uri}>" for uri in frame_uri_list])

        namespace_list = cls.get_default_namespace_list()

        binding_list = [
            Binding("?frame", "urn:hasFrame"),
            Binding("?slotEdge", "urn:hasSlotEdge"),
            Binding("?slot", "urn:hasSlot")
        ]

        slot_class_list = cls.get_slot_class_list()

        frame_query = f"""
            {{
                VALUES ?frameURI {{
                    {frame_uri_values}
                }}

                VALUES ?slotDataType {{
                    {slot_class_list}
                }}

                BIND(?frameURI AS ?frame)

                ?frame rdf:type haley-ai-kg:KGFrame .

                ?slotEdge vital-core:hasEdgeSource ?frame .
                ?slotEdge vital-core:hasEdgeDestination ?slot .
                ?slotEdge rdf:type haley-ai-kg:Edge_hasKGSlot .

                ?slot rdf:type ?slotType .
            }}

            FILTER(?slotType IN (?slotDataType))
            """

        construct_query = ConstructQuery(namespace_list, binding_list, frame_query)

        return construct_query

    @classmethod
    def generate_id_query(cls, frame_id: str) -> ConstructQuery:

        namespace_list = cls.get_default_namespace_list()

        binding_list = [
            Binding("?frame", "urn:hasFrame"),
            Binding("?slotEdge", "urn:hasSlotEdge"),
            Binding("?slot", "urn:hasSlot")
        ]

        root_binding = "?frame"

        slot_class_list = cls.get_slot_class_list()

        frame_query = f"""
                    {{
                        VALUES ?kg_identifier {{
                            "{frame_id}"^^<http://www.w3.org/2001/XMLSchema#string>
                        }}

                        VALUES ?slotDataType {{
                            {slot_class_list}
                        }}

                        BIND(?kg_identifier AS ?frame_id)

                        ?frame rdf:type haley-ai-kg:KGFrame .
                        ?frame haley-ai-kg:hasKGIdentifier ?frame_id .

                        ?slotEdge vital-core:hasEdgeSource ?frame .
                        ?slotEdge vital-core:hasEdgeDestination ?slot .
                        ?slotEdge rdf:type haley-ai-kg:Edge_hasKGSlot .

                        ?slot rdf:type ?slotType .
                    }}

                    FILTER(?slotType IN (?slotDataType))
                """

        construct_query = ConstructQuery(namespace_list, binding_list, frame_query, root_binding)

        return construct_query

    @classmethod
    def generate_id_list_query(cls, frame_id_list: List[str]) -> ConstructQuery:

        frame_id_values = "\n".join(
            [f'"{frame_id}"^^<http://www.w3.org/2001/XMLSchema#string>' for frame_id in frame_id_list])

        namespace_list = cls.get_default_namespace_list()

        binding_list = [
            Binding("?frame", "urn:hasFrame"),
            Binding("?slotEdge", "urn:hasSlotEdge"),
            Binding("?slot", "urn:hasSlot")
        ]

        slot_class_list = cls.get_slot_class_list()

        frame_query = f"""
                        {{
                            VALUES ?kg_identifier {{
                                {frame_id_values}
                        }}

                        VALUES ?slotDataType {{
                            {slot_class_list}
                        }}

                        BIND(?kg_identifier AS ?frame_id)

                        ?frame rdf:type haley-ai-kg:KGFrame .
                        ?frame haley-ai-kg:hasKGIdentifier ?frame_id .

                        ?slotEdge vital-core:hasEdgeSource ?frame .
                        ?slotEdge vital-core:hasEdgeDestination ?slot .
                        ?slotEdge rdf:type haley-ai-kg:Edge_hasKGSlot .

                        ?slot rdf:type ?slotType .
                    }}

                    FILTER(?slotType IN (?slotDataType))
                    """

        construct_query = ConstructQuery(namespace_list, binding_list, frame_query)

        return construct_query

    @classmethod
    def generate_criteria_query(cls,
                                criteria_list: List[KGSlotCriterion],
                                limit: int = 100, offset: int = 0) -> ConstructQuery:

        namespace_list = []
        binding_list = []
        query = ""

        construct_query = ConstructQuery(namespace_list, binding_list, query, limit, offset)

        return construct_query

    @classmethod
    def get_default_namespace_list(cls):
        namespace_list = [
            Ontology("vital-core", "http://vital.ai/ontology/vital-core#"),
            Ontology("vital", "http://vital.ai/ontology/vital#"),
            Ontology("vital-aimp", "http://vital.ai/ontology/vital-aimp#"),
            Ontology("haley", "http://vital.ai/ontology/haley"),
            Ontology("haley-ai-question", "http://vital.ai/ontology/haley-ai-question#"),
            Ontology("haley-ai-kg", "http://vital.ai/ontology/haley-ai-kg#")
        ]

        return namespace_list

    @classmethod
    def get_slot_class_list(cls):

        slot_class_list = """
            haley-ai-kg:KGSlot
            haley-ai-kg:KGAudioSlot
            haley-ai-kg:KGBooleanSlot
            haley-ai-kg:KGChoiceOptionSlot
            haley-ai-kg:KGChoiceSlot
            haley-ai-kg:KGCodeSlot
            haley-ai-kg:KGCurrencySlot
            haley-ai-kg:KGDateTimeSlot
            haley-ai-kg:KGDoubleSlot
            haley-ai-kg:KGEntitySlot
            haley-ai-kg:KGFileUploadSlot
            haley-ai-kg:KGGeoLocationSlot
            haley-ai-kg:KGImageSlot
            haley-ai-kg:KGIntegerSlot
            haley-ai-kg:KGJSONSlot
            haley-ai-kg:KGLongSlot
            haley-ai-kg:KGLongTextSlot
            haley-ai-kg:KGMultiChoiceOptionSlot
            haley-ai-kg:KGMultiChoiceSlot
            haley-ai-kg:KGMultiTaxonomyOptionSlot
            haley-ai-kg:KGMultiTaxonomySlot
            haley-ai-kg:KGPropertySlot
            haley-ai-kg:KGRunSlot
            haley-ai-kg:KGTaxonomyOptionSlot
            haley-ai-kg:KGTaxonomySlot
            haley-ai-kg:KGTextSlot
            haley-ai-kg:KGURISlot
            haley-ai-kg:KGVideoSlot
        """

        return slot_class_list




from kgraphservice.kg_comparator import KGComparator


class KGSlotCriterion:

    def __init__(self, slot_type_uri: str, comparator: KGComparator, value):
        self.slot_type_uri = slot_type_uri
        self.comparator = comparator
        self.value = value





from enum import Enum


class KGComparator(Enum):

    # replace values with URIs from ontology

    EXISTS = "exists"
    NOT_EXISTS = "not_exists"

    EQUALS = "equals"
    NOT_EQUAL_TO = "not_equal_to"

    LESS_THAN = "less_than"
    LESS_EQUAL = "less_equals"

    GREATER_THAN = "greater_than"
    GREATER_EQUAL = "greater_equal"

    # set membership
    CONTAINS = "contains"
    NOT_CONTAINS = "not_contains"



